@@ -45,7 +45,7 @@ struct obj_list {
 /* prototypes */
 TSS_RESULT	   obj_getTpmObject(UINT32, TSS_HOBJECT *);
 TSS_HOBJECT	   obj_GetPolicyOfObject(UINT32, UINT32);
-void		   obj_list_init();
+void		   __tspi_obj_list_init();
 TSS_HOBJECT	   obj_get_next_handle();
 TSS_RESULT	   obj_list_add(struct obj_list *, UINT32, TSS_FLAG, void *, TSS_HOBJECT *);
 TSS_RESULT	   obj_list_remove(struct obj_list *, void (*)(void *), TSS_HOBJECT, TSS_HCONTEXT);
@@ -67,7 +67,9 @@ TSS_RESULT	   obj_tcskey_get_pubkeyhash(TCS_KEY_HANDLE, BYTE *);
 #include "obj_policy.h"
 #include "obj_rsakey.h"
 #include "obj_encdata.h"
-#include "obj_daa.h"
+#include "obj_daacred.h"
+#include "obj_daaarakey.h"
+#include "obj_daaissuerkey.h"
 #include "obj_nv.h"
 #include "obj_delfamily.h"
 #include "obj_migdata.h"
@@ -80,7 +82,9 @@ PCRS_LIST_DECLARE_EXTERN;
 POLICY_LIST_DECLARE_EXTERN;
 RSAKEY_LIST_DECLARE_EXTERN;
 ENCDATA_LIST_DECLARE_EXTERN;
-DAA_LIST_DECLARE_EXTERN;
+DAACRED_LIST_DECLARE_EXTERN;
+DAAARAKEY_LIST_DECLARE_EXTERN;
+DAAISSUERKEY_LIST_DECLARE_EXTERN;
 NVSTORE_LIST_DECLARE_EXTERN;
 DELFAMILY_LIST_DECLARE_EXTERN;
 MIGDATA_LIST_DECLARE_EXTERN;
