@@ -29,7 +29,7 @@ struct platform_class
 struct tcsd_config
 {
 	int port;		/* port the TCSD will listen on */
-	int num_threads;	/* max number of threads the TCSD allows simultaneously */
+	unsigned int num_threads;	/* max number of threads the TCSD allows simultaneously */
 	char *system_ps_dir;	/* the directory the system PS file sits in */
 	char *system_ps_file;	/* the name of the system PS file */
 	char *firmware_log_file;/* the name of the firmware PCR event file */
@@ -46,9 +46,12 @@ struct tcsd_config
 	struct platform_class *host_platform_class; /* Host platform class of this TCS System */
 	struct platform_class *all_platform_classes;	/* List of platform classes
 							of this TCS System */
+	int disable_ipv4;
+	int disable_ipv6;
 };
 
-#define TCSD_CONFIG_FILE	ETC_PREFIX "/tcsd.conf"
+#define TCSD_DEFAULT_CONFIG_FILE	ETC_PREFIX "/tcsd.conf"
+extern char *tcsd_config_file;
 
 #define TSS_USER_NAME		"tss"
 #define TSS_GROUP_NAME		"tss"
@@ -60,12 +63,14 @@ struct tcsd_config
 #define TCSD_DEFAULT_KERNEL_LOG_FILE	"/sys/kernel/security/ima/binary_runtime_measurements"
 #define TCSD_DEFAULT_FIRMWARE_PCRS	0x00000000
 #define TCSD_DEFAULT_KERNEL_PCRS	0x00000000
+#define TCSD_DEFAULT_DISABLE_IPV4 0
+#define TCSD_DEFAULT_DISABLE_IPV6 0
 
 /* This will change when a system with more than 32 PCR's exists */
 #define TCSD_MAX_PCRS			32
 
 /* this is the 2nd param passed to the listen() system call */
-#define TCSD_MAX_SOCKETS_QUEUED		10
+#define TCSD_MAX_SOCKETS_QUEUED		50
 #define TCSD_TXBUF_SIZE			1024
 
 /* The Available Tcs Platform Classes */
@@ -96,6 +101,8 @@ struct tcg_platform_spec {
 #define TCSD_OPTION_REMOTE_OPS		0x0400
 #define TCSD_OPTION_EXCLUSIVE_TRANSPORT	0x0800
 #define TCSD_OPTION_HOST_PLATFORM_CLASS	0x1000
+#define TCSD_OPTION_DISABLE_IPV4 0x2000
+#define TCSD_OPTION_DISABLE_IPV6 0x4000
 
 #define TSS_TCP_RPC_MAX_DATA_LEN	1048576
 #define TSS_TCP_RPC_BAD_PACKET_TYPE	0x10000000
@@ -114,7 +121,9 @@ enum tcsd_config_option_code {
 	opt_remote_ops,
 	opt_exclusive_transport,
 	opt_host_platform_class,
-	opt_all_platform_classes
+	opt_all_platform_classes,
+	opt_disable_ipv4,
+	opt_disable_ipv6
 };
 
 struct tcsd_config_options {
@@ -134,7 +143,7 @@ struct tcsd_thread_data
 {
 	int sock;
 	UINT32 context;
-	THREAD_TYPE thread_id;
+	THREAD_TYPE *thread_id;
 	char *hostname;
 	struct tcsd_comm_data comm;
 };
