@@ -17,7 +17,21 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#if defined (HAVE_BYTEORDER_H)
+#include <sys/byteorder.h>
+#elif defined (HTOLE_DEFINED)
+#include <endian.h>
+#define LE_16 htole16
+#define LE_32 htole32
+#define LE_64 htole64
+#else
+#define LE_16(x) (x)
+#define LE_32(x) (x)
+#define LE_64(x) (x)
+#endif
 #include <assert.h>
+#include <fcntl.h>
+#include <limits.h>
 
 #include "trousers/tss.h"
 #include "trousers_types.h"
@@ -32,14 +46,18 @@
 int system_ps_fd = -1;
 MUTEX_DECLARE(disk_cache_lock);
 
+static struct flock fl;
+
 int
 get_file()
 {
-	int rc = 0;
-
+	int rc;
 	/* check the global file handle first.  If it exists, lock it and return */
 	if (system_ps_fd != -1) {
-		if ((rc = flock(system_ps_fd, LOCK_EX))) {
+		int rc = 0;
+
+		fl.l_type = F_WRLCK;
+		if ((rc = fcntl(system_ps_fd, F_SETLKW, &fl))) {
 			LogError("failed to get system PS lock: %s", strerror(errno));
 			return -1;
 		}
@@ -55,9 +73,10 @@ get_file()
 		return -1;
 	}
 
-	if ((rc = flock(system_ps_fd, LOCK_EX))) {
+	fl.l_type = F_WRLCK;
+	if ((rc = fcntl(system_ps_fd, F_SETLKW, &fl))) {
 		LogError("failed to get system PS lock of file %s: %s",
-				tcsd_options.system_ps_file, strerror(errno));
+			tcsd_options.system_ps_file, strerror(errno));
 		return -1;
 	}
 
@@ -68,10 +87,12 @@ int
 put_file(int fd)
 {
 	int rc = 0;
-
 	/* release the file lock */
-	if ((rc = flock(fd, LOCK_UN))) {
-		LogError("failed to unlock system PS file: %s", strerror(errno));
+	
+	fl.l_type = F_UNLCK;
+	if ((rc = fcntl(fd, F_SETLKW, &fl))) {
+		LogError("failed to unlock system PS file: %s",
+			strerror(errno));
 		return -1;
 	}
 
@@ -368,6 +389,12 @@ psfile_get_uuid_by_pub(int fd, TCPA_STORE_PUBKEY *pub, TSS_UUID **ret_uuid)
 
 		DBG_ASSERT(tmp->pub_data_size < 2048);
 
+		if (tmp->pub_data_size > sizeof(tmp_buffer)) {
+			LogError("Source buffer size too big! Size:  %d",
+				 tmp->pub_data_size);
+			MUTEX_UNLOCK(disk_cache_lock);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		}
 		/* read in the key */
                 if ((rc = read_data(fd, tmp_buffer, tmp->pub_data_size))) {
 			LogError("%s", __FUNCTION__);
@@ -431,6 +458,12 @@ psfile_get_key_by_pub(int fd, TCPA_STORE_PUBKEY *pub, UINT32 *size, BYTE **ret_k
                 }
 
 		DBG_ASSERT(tmp->pub_data_size < 2048);
+		if (tmp->pub_data_size > sizeof(tmp_buffer)) {
+			LogError("Source buffer size too big! Size:  %d",
+				 tmp->pub_data_size);
+			MUTEX_UNLOCK(disk_cache_lock);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		}
 
 		/* read in the key */
                 if ((rc = read_data(fd, tmp_buffer, tmp->pub_data_size))) {
@@ -456,6 +489,12 @@ psfile_get_key_by_pub(int fd, TCPA_STORE_PUBKEY *pub, UINT32 *size, BYTE **ret_k
                 }
 
 		DBG_ASSERT(tmp->blob_size < 4096);
+		if (tmp->blob_size > sizeof(tmp_buffer)) {
+			LogError("Blob size greater than 4096! Size:  %d",
+				 tmp->blob_size);
+			MUTEX_UNLOCK(disk_cache_lock);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		}
 
 		/* read in the key blob */
                 if ((rc = read_data(fd, tmp_buffer, tmp->blob_size))) {
@@ -565,28 +604,40 @@ psfile_write_key(int fd,
 	}
 
 	/* [UINT16   pub_data_size0  ] yes */
+	pub_key_size = LE_16(pub_key_size);
         if ((rc = write_data(fd, &pub_key_size, sizeof(UINT16)))) {
 		LogError("%s", __FUNCTION__);
                 goto done;
 	}
+	/* Swap it back for later */
+	pub_key_size = LE_16(pub_key_size);
 
 	/* [UINT16   blob_size0      ] yes */
+	key_blob_size = LE_16(key_blob_size);
         if ((rc = write_data(fd, &key_blob_size, sizeof(UINT16)))) {
 		LogError("%s", __FUNCTION__);
                 goto done;
 	}
+	/* Swap it back for later */
+	key_blob_size = LE_16(key_blob_size);
 
 	/* [UINT32   vendor_data_size0 ] yes */
+	vendor_size = LE_32(vendor_size);
         if ((rc = write_data(fd, &vendor_size, sizeof(UINT32)))) {
 		LogError("%s", __FUNCTION__);
                 goto done;
 	}
+	/* Swap it back for later */
+	vendor_size = LE_32(vendor_size);
 
 	/* [UINT16   cache_flags0    ] yes */
+	cache_flags = LE_16(cache_flags);
         if ((rc = write_data(fd, &cache_flags, sizeof(UINT16)))) {
 		LogError("%s", __FUNCTION__);
                 goto done;
 	}
+	/* Swap it back for later */
+	cache_flags = LE_16(cache_flags);
 
 	/* [BYTE[]   pub_data0       ] no */
         if ((rc = write_data(fd, (void *)key.pubKey.key, pub_key_size))) {
@@ -708,6 +759,7 @@ psfile_remove_key(int fd, struct key_disk_cache *c)
 	}
 
 	rc = read(fd, &num_keys, sizeof(UINT32));
+	num_keys = LE_32(num_keys);
 	if (rc != sizeof(UINT32)) {
 		LogError("read of %zd bytes: %s", sizeof(UINT32), strerror(errno));
 		return TCSERR(TSS_E_INTERNAL_ERROR);
@@ -722,6 +774,7 @@ psfile_remove_key(int fd, struct key_disk_cache *c)
 	/* decrement, then write back out to disk */
 	num_keys--;
 
+	num_keys = LE_32(num_keys);
 	if ((result = write_data(fd, (void *)&num_keys, sizeof(UINT32)))) {
 		LogError("%s", __FUNCTION__);
 		return result;
